package gol;

import java.awt.Point;
import java.util.*;
import static java.util.stream.Collectors.*;

public interface GameOfLife {
  public enum CellState { DEAD, ALIVE };
  
  public static CellState nextState(
    CellState currentState, long numberOfLiveNeighbor) {

    return numberOfLiveNeighbor == 3 || numberOfLiveNeighbor == 2 &&
      currentState == CellState.ALIVE ? CellState.ALIVE : CellState.DEAD;
  }
  
  public static List<Point> generateSignalsForAPosition(Point position) {
    int x = position.x;
    int y = position.y;
    
    return List.of(
      new Point(x - 1, y - 1), new Point(x - 1, y), new Point(x - 1, y + 1),
      new Point(x, y - 1), new Point(x, y + 1),
      new Point(x + 1, y - 1), new Point(x + 1, y), new Point(x + 1, y + 1));
  }
  
  public static List<Point> generateSignalsForPositions(
    List<Point> positions) {
    
    return positions.stream()
      .flatMap(position -> generateSignalsForAPosition(position).stream())
      .collect(toList());
  }
  
  public static Map<Point, Long> countSignals(List<Point> positions) {
    return positions.stream()
      .collect(groupingBy(position -> position, counting()));
  }
  
  public static List<Point> nextGeneration(List<Point> liveCells) {
    Map<Point, Long> signalsCount = countSignals(
      generateSignalsForPositions(liveCells));
    
    return signalsCount.keySet().stream()
      .filter(position -> nextState(
        liveCells.contains(position) ? CellState.ALIVE : CellState.DEAD,
          signalsCount.get(position)) == CellState.ALIVE)
      .collect(toList());
  }
  
  public static List<Point> boundsFor(List<Point> positions) {
    if(positions.size() == 0) {
      return List.of();
    }
    
    int leftX = Integer.MAX_VALUE;
    int leftY = Integer.MAX_VALUE;
    int rightX = Integer.MIN_VALUE;
    int rightY = Integer.MIN_VALUE;                                                
                                   
    for(Point position : positions) {
      if(position.x < leftX) leftX = position.x;
      if(position.y < leftY) leftY = position.y;
      if(position.x > rightX) rightX = position.x;
      if(position.y > rightY) rightY = position.y;
    }
    
    return List.of(new Point(leftX, leftY), new Point(rightX, rightY));
  }
}







